/****************************************************************************
 * Copyright  2003-2007 Dorian C. Arnold, Philip C. Roth, Barton P. Miller *
 *                  Detailed MRNet usage rights in "LICENSE" file.          *
 ****************************************************************************/

#if !defined(__remotenode_h) 
#define __remotenode_h 1

#include <map>

#include "CommunicationNode.h"
#include "Message.h"
#include "Error.h"
#include "mrnet/Types.h"
#include "xplat/Thread.h"
#include "xplat/Monitor.h"
#include "xplat/Mutex.h"

namespace MRN
{

class ChildNode;
class ParentNode;
class Network;

class Packet;


class RemoteNode:public CommunicationNode, public Error {
    friend class Network;
 public:
    static RemoteNodePtr NullRemoteNode;
    static void * recv_thread_main(void * arg);
    static void * send_thread_main(void * arg);

    int connect_DataSocket( void );
    int connect_EventSocket( void );

    int new_InternalNode(int listening_sock_fd,
                         std::string parent_hostname, Port parent_port,
                         std::string commnode) const;
    int new_Application(int listening_sock_fd,
                        std::string parent_hostname, Port parent_port,
                        Rank be_rank,
                        std::string &cmd, std::vector <std::string> &args) const;

    static int connect_to_backend( int listening_sock, Rank* rank );
    int connect_to_leaf( Rank r );

    int send(const PacketPtr ) const ;
    int flush( bool ignore_threads = false ) const ;
    int recv(std::list <PacketPtr> &) const; //blocking recv
    bool has_data() const;
    bool is_backend() const;
    bool is_internal() const;
    bool is_upstream() const;

    void set_DataSocketFd( int isock ) { _data_sock_fd = isock; }
    int get_DataSocketFd( void ) const { return _data_sock_fd; }
    int get_EventSocketFd( void ) const { return _event_sock_fd; }

    static void set_BlockingTimeOut( int _timeout );
    static int get_BlockingTimeOut( void );
    int start_CommunicationThreads( void );

    static void cancel_DownstreamSendThreads( void );

 private:
    Network * _network;
    int _data_sock_fd;
    int _event_sock_fd;
    bool _is_internal_node;
    bool _is_upstream;

    enum {MRN_MESSAGEOUT_NONEMPTY};
    mutable XPlat::Monitor _msg_out_sync;
    mutable Message _msg_out;
    mutable Message _msg_in;


    XPlat::Thread::Id recv_thread_id, send_thread_id;

    RemoteNode( Network *, std::string const& ihostname, Port iport, Rank irank,
                bool iis_upstream, bool iis_internal );

    static int poll_timeout;
    static XPlat::Mutex poll_timeout_mutex;
};

} // namespace MRN

#endif /* __remotenode_h  */
